// -*- C++ -*-
/*!
 * @file  HumanDetection_LRF.h
 * @brief HumanDetection_LRF compornent
 * @date $Date$ 
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$ 
 */
#ifndef HUMANDETECTION_LRF_H
#define HUMANDETECTION_LRF_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "LRFProcessSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "MyDataTypeStub.h"

// </rtc-template>

using namespace RTC;

#include "type.h"
#include "viewgui.h"
#include<math.h>
//#include "UrgProcess.h"
#include <iostream>
using namespace std;

class HumanDetection_LRF  : public RTC::DataFlowComponentBase
{
 public:
  HumanDetection_LRF(RTC::Manager* manager);
  ~HumanDetection_LRF();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedLongSeq m_UrgData;
  InPort<TimedLongSeq> m_UrgDataIn;
  TimedXYDouble m_VisionPos;
  InPort<TimedXYDouble> m_VisionPosIn;
  TimedOdometry m_TargetOdometry;
  InPort<TimedOdometry> m_TargetOdometryIn;
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedXYDouble m_HumanPosXY;
  OutPort<TimedXYDouble> m_HumanPosXYOut;
  TimedOdometry m_HumanPos;
  OutPort<TimedOdometry> m_HumanPosOut;
  TimedObstacles m_Obstacle;
  OutPort<TimedObstacles> m_ObstacleOut;
  TimedXYDoubleSeq m_HumanObject;
  OutPort<TimedXYDoubleSeq> m_HumanObjectOut;
  TimedXYDoubleSeq m_ObstacleObject;
  OutPort<TimedXYDoubleSeq> m_ObstacleObjectOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_TrackingPortPort;
  RTC::CorbaPort m_LRFInfoPort;

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  LRFTrackingSVC_impl m_LRFTracking;

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  RTC::CorbaConsumer<LRFInfo> m_LRF;

  // </rtc-template>

 private:
	bool *port_connect_check;
    long int *g_distance;
	double *point_x,*point_y;
	double *obstacle_x,*obstacle_y;
	int obstacle_num;
	double *target_px,*target_py;
	int target_pnum;

     double target_x,target_y;
     double vision_x,vision_y;

	int lrf_scanpoint;
	double lrf_resolusion;
	double lrf_offset;

	Graphics GL;

};


extern "C"
{
  void HumanDetection_LRFInit(RTC::Manager* manager);
};

#endif // HUMANDETECTION_LRF_H

