/*!
 * @brief Obstacle module
 * $Date$
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$
 */


#include "viewgui.h"
#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glui.h>
#include <time.h>
#include <sys/time.h>
#include <iostream>
#include <math.h>
using namespace std;

double start_x=0.0,start_y=-0.8;

//void PathPlanning(point goal,point robot_position);
double input_x,input_y;
//ウィンドウのサイズ	
const int width = 600, height = 600;
 
//各種変数宣言
int xBegin, yBegin;
int mButton;
float distance_gl, twist, elevation, azimuth;
float xOrig , yOrig, zOrig;
int display_flag;

double goal_urg_x;
double goal_urg_y;
bool id_flag;

 int Graphics::p1num,Graphics::p2num;

double Graphics::px1[NUMBER],Graphics::py1[NUMBER];
double Graphics::px2[NUMBER],Graphics::py2[NUMBER];
double Graphics::xx[NUMBER],Graphics::yy[NUMBER];
int Graphics::POINTS;

Graphics::Graphics(){
xOrig = 0.0, yOrig = 04.0, zOrig=0.0; //始点中心

	//DEFAULT IS TOP-URG SENSOR
	CYCLE=1440.0;
	point=1081.0;
	POINTS=1081;
	resolusion=PI/1440.0;

    gui_start();
}

Graphics::~Graphics(){
    gui_end();
}


	
void Graphics::SetObstacle(int Pnum,double *px,double *py){
	p2num=Pnum;
	if((Pnum<0)||(Pnum>NUMBER)) {
	cout<<"Object Scan Data Error"<<endl;
	return;
	}
	
	for(int i=0;i<POINTS;i++){	
		px2[i]=px[i];
		py2[i]=py[i];
	}

}

void Graphics::SetObject(int Pnum,double *px,double *py){
	p1num=Pnum;
	if((Pnum<0)||(Pnum>NUMBER)) {
	cout<<"Object Scan Data Error"<<endl;
	return;
	}
	
	for(int i=0;i<POINTS;i++){	
		px1[i]=px[i];
		py1[i]=py[i];
	}
}

void Graphics::SetObjectXY(double px,double py){
        if(id_flag==false)id_flag=true;
        else if(id_flag==true)id_flag=false;
	goal_urg_x=px;
	goal_urg_y=py;
        
}

void Graphics::Input(bool& id,double &goal_x,double &goal_y){
		id=id_flag;
		goal_x=goal_urg_x;
		goal_y=goal_urg_y;

}


void Graphics::SetLRF(int Pnum,double *px,double *py){

	POINTS=Pnum;
	if((Pnum<0)||(Pnum>NUMBER)) {
	cout<<"LRF Scan Data Error"<<endl;
	return;
	}
	
	for(int i=0;i<POINTS;i++){	
		xx[i]=px[i];
		yy[i]=py[i];
	}

}




void Graphics::draw() {
    glPointSize(3);
    glBegin(GL_POINTS);
    glColor3f( 0.0, 1.0, 0.0 );
    for(int k=0;k<POINTS;k++){
//			glVertex2f(xx[k],yy[k]);
    }
    glEnd();


    glPointSize(3);
    glBegin(GL_POINTS);
  	    glColor3f( 0.0, 1.0, 0.0 );
    for(int k=0;k<p1num;k++){
			glVertex2f(px1[k],py1[k]);
    }
    glEnd();

    glPointSize(3);
    glBegin(GL_POINTS);
      glColor3f( 1.0, 0.0,0.0 );
    for(int k=0;k<p2num;k++){
			glVertex2f(px2[k],py2[k]);
    }
    glEnd();

  glPointSize(15);
    glBegin(GL_POINTS);
  	    glColor3f( 0.0, 1.0, 0.0 );
		glVertex2f(goal_urg_x,goal_urg_y);
    glEnd();

   double x0,x1,y0,y1;
    //四角形を表示
    glColor3f(0,1,0);
    glLineWidth(3);
    double x=goal_urg_x;
    double y=goal_urg_y;
    x0=x-0.30;
    x1=x+0.30;
    y0=y-0.30;
    y1=y+0.30;
    glBegin(GL_LINE_LOOP);
    glVertex2d(x0, y0);
    glVertex2d(x1, y0);
    glVertex2d(x1, y1);
    glVertex2d(x0, y1);
    glEnd();

} 



void Graphics::display()
{
	usleep(10*1000);

    glClearColor(1 , 1 , 1 , 1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);    
    glEnable(GL_DEPTH_TEST);
    glPushMatrix ();
    polarview();
	draw();

    glPopMatrix();
    glutSwapBuffers(); 
    glutPostRedisplay();

}

void Graphics::idle(void)
{    
    glutPostRedisplay();
    usleep(10*1000);
}

void Graphics::myInit(char *progname)
{
    float aspect = (float) width / (float) height;
 
    glutInitWindowPosition(0, 0);
    glutInitWindowSize( width, height);
//    glutInitDisplayMode( GLUT_RGBA| GLUT_DEPTH);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA| GLUT_DEPTH);
//    glutInitDisplayMode(GLUT_RGBA);
    glutCreateWindow(progname);
  //  glui_id = glutCreateWindow( progname );
    glClearColor (1, 1, 1, 1.0);
 

    glutKeyboardFunc( myKbd );
    glutSpecialFunc( mySkey );
 
    glutMouseFunc(myMouse);
    glutMotionFunc(myMotion);
    resetview();
 
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, aspect, 0.001, 100.0);//画角等視野の設定
    glMatrixMode(GL_MODELVIEW);
}

void Graphics::reshape(int ,int)
{
 //   glutReshapeWindow( WIDTH, HEIGHT );
}

int Graphics::GUImain(){ 

    display_flag=3;   
    int argc= 3;
    char *argv[]    = {"program", "-display", ":0.0"};
    glutInit ( &argc, argv );
    myInit("HumanDetectionLRF");
   // glui_init();
    glutDisplayFunc(display);
    glutReshapeFunc(reshape);
    glClearColor(1.0, 1.0, 1.0, 1.0);

    glutIdleFunc(idle);
    glutMainLoop();


    return 0;
}


void Graphics::gui_start(){
    th1 = SDL_CreateThread((int (*)(void *))GUImain,this);
}


void Graphics::gui_end(){
   SDL_KillThread(th1);
}

double Graphics::timer_ms() {
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return (tv.tv_sec + tv.tv_usec * 1e-6)*1e3;
} 


void Graphics::myMouse( int button, int state, int x, int y )
{
    id_flag=false;
    if (state == GLUT_DOWN) {
        switch(button) {
        case GLUT_LEFT_BUTTON:
            mButton = button;

	    break;
        case GLUT_MIDDLE_BUTTON:

            break;
        case GLUT_RIGHT_BUTTON:
  	    mButton = button;
	    id_flag=true; 
            break;
        }
        xBegin = x;
        yBegin = y;
    }
    if (state == GLUT_UP) {
	switch(button) {
	case GLUT_RIGHT_BUTTON:
	    id_flag=false; 
	    break;
	}
    }
}

void Graphics::myMotion( int x, int y )
{
    int xDisp, yDisp;
 
    xDisp = x - xBegin;
    yDisp = y - yBegin;
 
    switch (mButton) {
    case GLUT_LEFT_BUTTON:
	//	  azimuth -= (float) xDisp/2.0;
	//	  elevation -= (float) yDisp/2.0;
	xOrig += (float) xDisp/50.0;
	zOrig -= (float) yDisp/50.0;

	break;
    case GLUT_RIGHT_BUTTON:
	goal_urg_x += (float) xDisp/50.0;
	goal_urg_y -= (float) yDisp/50.0;
	//	distance_gl += (float) xDisp/40.0;
	//		yOrig -= (float) xDisp/50.0;
	break;
    }
    xBegin = x;
    yBegin = y;
    glutPostRedisplay();
}
 
 
void Graphics::myKbd( unsigned char key, int x, int y )
{
    switch( key ) {
    case 'q':    //終了
        //return 0;
	//scan_flg=0;
	//        exit(1);
        break;
    case 'c':    //視点を中心に移動
        distance_gl = 1.0;
        twist = 0.0;
        elevation = 0.0;
        azimuth = 0.0;
        break;
    case 'r':    //視点をリセット
	xOrig = 0.0, yOrig = 0.0, zOrig=0.0; 
        //resetview();
        break;
    case 'w':    
	goal_urg_y+=0.2;
        break;    
    case 's':    
	goal_urg_y-=0.2;
        break;
    case 'a':    
	goal_urg_x-=0.2;
        break;
    case 'd':    
	goal_urg_x+=0.2;
        break;
    case 'z':    
		if(id_flag==true) id_flag=false;	
		else if(id_flag==false) id_flag=true;
	//	cout<<id_flag<<"\n";
        break;
    }
    glutPostRedisplay();
} 
 
 
void Graphics::mySkey( int key, int x, int y )
{
    switch( key ) {
    case GLUT_KEY_LEFT:
        xOrig -= 0.2;
        break;
    case GLUT_KEY_RIGHT:
        xOrig += 0.2;
        break;
    case GLUT_KEY_UP:
        zOrig += 0.2;
        break; 
    case GLUT_KEY_DOWN:
        zOrig -= 0.2;
        break;
    case GLUT_KEY_PAGE_UP:
        yOrig += 0.2;
        break;
    case GLUT_KEY_PAGE_DOWN:
        yOrig -= 0.2;
        break;
 
    }
    glutPostRedisplay();
}
 

void Graphics::resetview( void )
{
    distance_gl = 10.0;
    twist = 0.0;
    elevation = 0.0;
    azimuth = 0.0;
}



void Graphics::polarview( void )
{
    //マウスで移動
    glTranslatef( 0.0, 0.0, -distance_gl);
    glRotatef( -twist, 0.0, 0.0, 1.0);
    glRotatef( -elevation, 1.0, 0.0, 0.0);
    glRotatef( -azimuth, 0.0, 1.0, 0.0);
    //キーボードで移動
    glTranslatef(xOrig,zOrig ,yOrig);
}
 
